package cn.com.qmth.stmms.api.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.com.qmth.stmms.admin.utils.SessionExamUtils;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.fasterxml.jackson.databind.ObjectMapper;

import cn.com.qmth.stmms.api.exception.ApiException;
import cn.com.qmth.stmms.biz.basic.service.PrivilegeService;
import cn.com.qmth.stmms.biz.basic.service.RolePrivilegeService;
import cn.com.qmth.stmms.biz.exam.service.SubjectUserService;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.authorization.AuthorizationService;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.domain.AuthInfo;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.session.service.SessionService;
import cn.com.qmth.stmms.common.signature.SignatureInfo;
import cn.com.qmth.stmms.common.signature.SignatureType;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;

import java.util.Map;

/**
 * API接口访问拦截器
 *
 * @author LS
 */
public class ApiInterceptor extends HandlerInterceptorAdapter {

    protected static final Logger log = LoggerFactory.getLogger(ApiInterceptor.class);

    @Autowired
    private AuthorizationService authorizationService;

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private UserService userService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private SubjectUserService subjectUserService;

    @Autowired
    private RolePrivilegeService rolePrivilegeService;

    @Autowired
    private PrivilegeService privilegeService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        buildSession(request);
        HandlerMethod method = (HandlerMethod) handler;
        return validate(request, response, method.getMethodAnnotation(RoleRequire.class));
    }

    private boolean validate(HttpServletRequest request, HttpServletResponse response, RoleRequire authConfig) {
        // if (authConfig != null && authConfig.value().length > 0) {
        AuthInfo simple;
        // 优先按签名模式解析
        SignatureInfo info = authorizationService.buildSignature(request);
        if (info != null) {
            if (info.getType() == SignatureType.SECRET) {
                School school = schoolService.findByAccessKey(info.getInvoker());
                if (school == null || !info.validate(school.getAccessSecret())) {
                    throw ApiException.SIGNATURE_INVALID;
                }
                if (!school.isEnable()) {
                    throw ApiException.USER_DISABLED;
                }
                if (authConfig != null && authConfig.value().length > 0 && !matchRole(authConfig, Role.SCHOOL_DEV)) {
                    throw ApiException.USER_ROLE_INVALID;
                }
                return buildApiUser(request, response, school);
            } else if (info.getType() == SignatureType.TOKEN) {
                User user = userService.findByLoginName(info.getInvoker());
                if (user == null || !info.validate(user.getAccessToken())) {
                    try {
                        ObjectMapper mapper = new ObjectMapper();
                        log.warn("api token signature invalud, signature={}, user={}", mapper.writeValueAsString(info),
                                user != null ? mapper.writeValueAsString(user) : "{}");
                    } catch (Exception ignored) {
                    }
                    throw ApiException.SIGNATURE_INVALID;
                }
                if (!user.isEnable()) {
                    throw ApiException.USER_DISABLED;
                }
                if (authConfig != null && authConfig.value().length > 0 && !matchRole(authConfig, user.getRole())) {
                    throw ApiException.USER_ROLE_INVALID;
                }
                return buildApiUser(request, response, user);
            } else {
                throw ApiException.SIGNATURE_INVALID;
            }
        }
        // 再尝试按简单模式解析
        else if ((simple = authorizationService.buildAuthInfo(request)) != null) {
            User user = userService.findByLoginName(simple.getLoginname());
            if (user == null) {
                throw ApiException.SIGNATURE_INVALID;
            }
            if (!EncryptUtils.md5(simple.getPassword()).equals(user.getPassword())) {
                throw ApiException.PASSWORD_ERROR;
            }
            if (!user.isEnable()) {
                throw ApiException.USER_DISABLED;
            }
            if (authConfig != null && authConfig.value().length > 0 && !matchRole(authConfig, user.getRole())) {
                throw ApiException.USER_ROLE_INVALID;
            }
            return buildApiUser(request, response, user);
        } else {
            throw ApiException.AUTHORIZATION_UNEXIST;
        }
        // }
        // return true;
    }

    private boolean matchRole(RoleRequire authConfig, Role role) {
        int count = 0;
        for (Role r : authConfig.value()) {
            if (r == role) {
                count++;
            }
        }
        if (authConfig.nand()) {
            return count > 0;
        } else {
            return count == authConfig.value().length;
        }
    }

    private boolean buildApiUser(HttpServletRequest request, HttpServletResponse response, School school) {
        RequestUtils.setApiUser(request, new ApiUser(school));
        sessionService.put(request, response, RequestUtils.getSession(request));
        return true;
    }

    private boolean buildApiUser(HttpServletRequest request, HttpServletResponse response, User user) {
        if (user == null) {
            return false;
        }
        Integer examId = SessionExamUtils.getExamId(request);
        String uri = request.getRequestURI();
        if ((examId == null || examId <= 0) && !uri.startsWith("/api/admin/exam/list")) {
            return false;
        }
        ApiUser wu = new ApiUser(user);
        if (wu.getRole() == Role.SYS_ADMIN) {
            RequestUtils.setApiUser(request, wu);
            sessionService.put(request, response, RequestUtils.getSession(request));
            return true;
        }
        if (wu.getRole() == Role.SUBJECT_HEADER || wu.getRole() == Role.INSPECTOR || wu.getRole() == Role.COLLEGE_ADMIN
                || wu.getRole() == Role.SCHOOL_VIEWER) {
            wu.setSubjectCodeSet(subjectUserService.findSubjectCode(wu.getId()));
        }
        RequestUtils.setRolePrivilegeCodes(request,
                rolePrivilegeService.getRoleCodes(wu.getUser().getSchoolId(), wu.getRole()));
        RequestUtils.setPrivileges(request, privilegeService.getMenuPrivileges());
        RequestUtils.setApiUser(request, wu);
        sessionService.put(request, response, RequestUtils.getSession(request));
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
            throws Exception {
        try {
            sessionService.put(request, response, RequestUtils.getSession(request));
        } catch (Exception e) {
            log.error("stmms session save faile!", e);
        }
    }

    protected void buildSession(HttpServletRequest request) {
        StmmsSession session = sessionService.get(request);
        RequestUtils.setSession(request, session);
        for (Map.Entry<String, String> entry : session.getParameters()) {
            request.setAttribute(entry.getKey(), entry.getValue());
        }
    }

}
